/*
 * Copyright 2014 CyberVision, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kaaproject.kaa.server.admin.client.mvp.place;

import org.kaaproject.kaa.server.admin.client.util.Utils;

import com.google.gwt.place.shared.PlaceTokenizer;
import com.google.gwt.place.shared.Prefix;
import com.google.web.bindery.event.shared.EventBus;

public class LogAppenderPlace extends LogAppendersPlace {

    protected String appenderId;

    public LogAppenderPlace(String applicationId, String appenderId) {
        super(applicationId);
        this.appenderId = appenderId;
    }

    public String getAppenderId() {
        return appenderId;
    }

    @Override
    public String getName() {
        return Utils.constants.logAppender();
    }

    @Prefix(value = "appender")
    public static class Tokenizer implements PlaceTokenizer<LogAppenderPlace>, PlaceConstants {

        @Override
        public LogAppenderPlace getPlace(String token) {
            PlaceParams.paramsFromToken(token);
            return new LogAppenderPlace(PlaceParams.getParam(APPLICATION_ID), PlaceParams.getParam(APPENDER_ID));
        }

        @Override
        public String getToken(LogAppenderPlace place) {
            PlaceParams.clear();
            PlaceParams.putParam(APPLICATION_ID, place.getApplicationId());
            PlaceParams.putParam(APPENDER_ID, place.getAppenderId());
            return PlaceParams.generateToken();
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        LogAppenderPlace other = (LogAppenderPlace) obj;
        if (appenderId == null) {
            if (other.appenderId != null)
                return false;
        } else if (!appenderId.equals(other.appenderId))
            return false;
        return true;
    }

    @Override
    public boolean isLeaf() {
        return true;
    }

    @Override
    public TreePlaceDataProvider getDataProvider(EventBus eventBus) {
        return null;
    }

    @Override
    public TreePlace createDefaultPreviousPlace() {
        return new LogAppendersPlace(applicationId);
    }
}
