/*
 * Copyright 2014 CyberVision, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


include "shared.thrift"
include "cli.thrift"

namespace java org.kaaproject.kaa.server.common.thrift.gen.operations
namespace cpp kaa

typedef shared.ObjectId id

typedef double probability
typedef string uri

typedef binary endpoint_id
typedef string user_id
typedef string tenant_id
typedef string endpoint_class_family_id
typedef string application_token

/**
* This is enum of changes type. The same as org.kaaproject.kaa.common.dto.ChangeType
**/
enum Operation {
      UPDATE = 1,
      DELETE = 2,
      INSERT = 3,
      ADD_TOPIC = 4,
      REMOVE_TOPIC = 5,
      UPDATE_WEIGHT = 6,
      ADD_CONF = 7,
      REMOVE_CONF = 8,
      ADD_PROF = 9,
      REMOVE_PROF = 10,
      REMOVE_GROUP = 11,
      REMOVE_CONF_VERSION = 12,
      REMOVE_PROF_VERSION = 13,
      REMOVE_NOTIFICATION_VERSION = 14
      ADD_LOG_APPENDER = 15
      REMOVE_LOG_APPENDER = 16
      UPDATE_LOG_APPENDER = 17
}

/**
* Enum defines route update operation type
*/
enum EventRouteUpdateType {
      ADD = 1;
      DELETE = 2;
      UPDATE = 3
}

/**
* Defines types of messages, all pass through one interface and demultiplex by this enum
*/
enum EventMessageType {
      ROUTE_UPDATE = 1;
      USER_ROUTE_INFO = 2;
      EVENT = 3
}

struct Notification {
  1: id appId
  2: shared.Integer appSeqNumber
  3: id groupId
  4: shared.Integer groupSeqNumber
  5: id profileFilterId
  6: shared.Integer profileFilterSeqNumber
  7: id configurationId
  8: shared.Integer configurationSeqNumber
  9: id notificationId
  10: id unicastNotificationId
  11: id topicId
  12: binary keyHash
  13: Operation op
  14: id appenderId
}

struct RedirectionRule {
  1: uri dnsName
  2: shared.Long ruleId
  3: probability redirectionProbability
  4: shared.Long ruleTTL
}

struct EventClassFamilyVersion {
  1: endpoint_class_family_id endpointClassFamilyId
  2: shared.Integer  endpointClassFamilyVersion
}

struct RouteInfo {
  1: EventRouteUpdateType updateType
  2: list<EventClassFamilyVersion> eventClassFamilyVersion
  3: application_token applicationToken
  4: endpoint_id endpointId
}

struct EventRoute {
  1: user_id userId
  2: tenant_id tenantId
  3: list<RouteInfo> routeInfo
  4: string operationsServerId
}

struct EndpointEvent {
  1: string uuid
  2: endpoint_id sender
  3: binary eventData
  4: shared.Long createTime
  5: shared.Integer version
}

struct RouteAddress {
  1: endpoint_id endpointKey
  2: application_token applicationToken
  3: string operationsServerId
}

struct Event {
  1: user_id userId
  2: tenant_id tenantId
  3: EndpointEvent endpointEvent
  4: RouteAddress routeAddress
}

struct UserRouteInfo {
  1: user_id userId
  2: tenant_id tenantId
  3: string operationsServerId
  4: EventRouteUpdateType updateType
}

struct EventMessage {
  1: EventMessageType type
  2: shared.Long eventId
  3: Event event
  4: EventRoute route
  5: UserRouteInfo userRoute
}

service OperationsThriftService extends cli.CliThriftService{

/**
*   Application Sequence Numbers
*/
  void onNotification(1: Notification notification);


/**
*  Set redirection rule for Operations server
*/
  void setRedirectionRule(1: RedirectionRule redirectionRule);


/**
*  Interface to send unified event messages
*/
  void sendEventMessage(1: list<EventMessage> messages);

}
