/*
 * Copyright 2014 CyberVision, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kaaproject.kaa.server.operations.service.http;

import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.util.concurrent.DefaultEventExecutorGroup;

import java.util.UUID;

import org.kaaproject.kaa.server.common.server.http.CommandProcessor;
import org.kaaproject.kaa.server.common.server.http.DefaultHttpServerInitializer;
import org.kaaproject.kaa.server.operations.service.akka.AkkaService;
import org.kaaproject.kaa.server.operations.service.http.handler.AkkaHttpHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * The Class OperationsServerInitializer.
 */
@Component
public class HttpServerInitializer extends DefaultHttpServerInitializer {

    /** The Constant logger. */
    private static final Logger LOG = LoggerFactory.getLogger(HttpServerInitializer.class);

    @Autowired
    AkkaService akkaService;

    /**
     * Instantiates a new end point server initializer.
     */
    public HttpServerInitializer() {
        super();
        LOG.info("Operations Server Initializer ...");
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.kaaproject.kaa.server.common.http.server.DefaultServerInitializer
     * #getMainHandler(java.util.UUID)
     */
    @Override
    protected SimpleChannelInboundHandler<CommandProcessor> getMainHandler(UUID uuid) {
        return new AkkaHttpHandler(uuid, akkaService, new DefaultEventExecutorGroup(1));
    }

}
